@@ -19,8 +19,8 @@
 #include <sstream>
 #include "obs-config.h"
 #include "obs-app.hpp"
+#include "qt-wrappers.hpp"
 #include "platform.hpp"
-using namespace std;
 
 #include <util/windows/win-version.h>
 #include <util/platform.h>
@@ -30,7 +30,6 @@ using namespace std;
 #include <shellapi.h>
 #include <shlobj.h>
 #include <Dwmapi.h>
-#include <psapi.h>
 #include <mmdeviceapi.h>
 #include <audiopolicy.h>
 
@@ -38,14 +37,16 @@ using namespace std;
 #include <util/windows/HRError.hpp>
 #include <util/windows/ComPtr.hpp>
 
-static inline bool check_path(const char* data, const char *path,
-		string &output)
+using namespace std;
+
+static inline bool check_path(const char *data, const char *path,
+			      string &output)
 {
 	ostringstream str;
 	str << path << data;
 	output = str.str();
 
-	printf("Attempted path: %s\n", output.c_str());
+	blog(LOG_DEBUG, "Attempted path: %s", output.c_str());
 
 	return os_file_exists(output.c_str());
 }
@@ -58,18 +59,13 @@ bool GetDataFilePath(const char *data, string &output)
 	return check_path(data, OBS_DATA_PATH "/obs-studio/", output);
 }
 
-bool InitApplicationBundle()
-{
-	return true;
-}
-
 string GetDefaultVideoSavePath()
 {
 	wchar_t path_utf16[MAX_PATH];
-	char    path_utf8[MAX_PATH]  = {};
+	char path_utf8[MAX_PATH] = {};
 
 	SHGetFolderPathW(NULL, CSIDL_MYVIDEO, NULL, SHGFP_TYPE_CURRENT,
-			path_utf16);
+			 path_utf16);
 
 	os_wcs_to_utf8(path_utf16, wcslen(path_utf16), path_utf8, MAX_PATH);
 	return string(path_utf8);
@@ -80,18 +76,18 @@ static vector<string> GetUserPreferredLocales()
 	vector<string> result;
 
 	ULONG num, length = 0;
-	if (!GetUserPreferredUILanguages(MUI_LANGUAGE_NAME, &num,
-				nullptr, &length))
+	if (!GetUserPreferredUILanguages(MUI_LANGUAGE_NAME, &num, nullptr,
+					 &length))
 		return result;
 
 	vector<wchar_t> buffer(length);
 	if (!GetUserPreferredUILanguages(MUI_LANGUAGE_NAME, &num,
-				&buffer.front(), &length))
+					 &buffer.front(), &length))
 		return result;
 
 	result.reserve(num);
 	auto start = begin(buffer);
-	auto end_  = end(buffer);
+	auto end_ = end(buffer);
 	decltype(start) separator;
 	while ((separator = find(start, end_, 0)) != end_) {
 		if (result.size() == num)
@@ -161,31 +157,18 @@ uint32_t GetWindowsVersion()
 	return ver;
 }
 
-void SetAeroEnabled(bool enable)
+uint32_t GetWindowsBuild()
 {
-	static HRESULT (WINAPI *func)(UINT) = nullptr;
-	static bool failed = false;
+	static uint32_t build = 0;
 
-	if (!func) {
-		if (failed) {
-			return;
-		}
+	if (build == 0) {
+		struct win_version_info ver_info;
 
-		HMODULE dwm = LoadLibraryW(L"dwmapi");
-		if (!dwm) {
-			failed = true;
-			return;
-		}
-
-		func = reinterpret_cast<decltype(func)>(GetProcAddress(dwm,
-						"DwmEnableComposition"));
-		if (!func) {
-			failed = true;
-			return;
-		}
+		get_win_ver(&ver_info);
+		build = ver_info.build;
 	}
 
-	func(enable ? DWM_EC_ENABLECOMPOSITION : DWM_EC_DISABLECOMPOSITION);
+	return build;
 }
 
 bool IsAlwaysOnTop(QWidget *window)
@@ -198,7 +181,7 @@ void SetAlwaysOnTop(QWidget *window, bool enable)
 {
 	HWND hwnd = (HWND)window->winId();
 	SetWindowPos(hwnd, enable ? HWND_TOPMOST : HWND_NOTOPMOST, 0, 0, 0, 0,
-			SWP_NOMOVE | SWP_NOSIZE | SWP_NOACTIVATE);
+		     SWP_NOMOVE | SWP_NOSIZE | SWP_NOACTIVATE);
 }
 
 void SetProcessPriority(const char *priority)
@@ -209,9 +192,13 @@ void SetProcessPriority(const char *priority)
 	if (strcmp(priority, "High") == 0)
 		SetPriorityClass(GetCurrentProcess(), HIGH_PRIORITY_CLASS);
 	else if (strcmp(priority, "AboveNormal") == 0)
-		SetPriorityClass(GetCurrentProcess(), ABOVE_NORMAL_PRIORITY_CLASS);
+		SetPriorityClass(GetCurrentProcess(),
+				 ABOVE_NORMAL_PRIORITY_CLASS);
 	else if (strcmp(priority, "Normal") == 0)
 		SetPriorityClass(GetCurrentProcess(), NORMAL_PRIORITY_CLASS);
+	else if (strcmp(priority, "BelowNormal") == 0)
+		SetPriorityClass(GetCurrentProcess(),
+				 BELOW_NORMAL_PRIORITY_CLASS);
 	else if (strcmp(priority, "Idle") == 0)
 		SetPriorityClass(GetCurrentProcess(), IDLE_PRIORITY_CLASS);
 }
@@ -224,18 +211,39 @@ void SetWin32DropStyle(QWidget *window)
 	SetWindowLongPtr(hwnd, GWL_EXSTYLE, ex_style);
 }
 
+bool SetDisplayAffinitySupported(void)
+{
+	static bool checked = false;
+	static bool supported;
+
+	/* this has to be version gated as setting WDA_EXCLUDEFROMCAPTURE on
+	   older Windows builds behaves like WDA_MONITOR (black box) */
+
+	if (!checked) {
+		if (GetWindowsVersion() > 0x0A00 ||
+		    GetWindowsVersion() == 0x0A00 && GetWindowsBuild() >= 19041)
+			supported = true;
+		else
+			supported = false;
+
+		checked = true;
+	}
+
+	return supported;
+}
+
 bool DisableAudioDucking(bool disable)
 {
-	ComPtr<IMMDeviceEnumerator>   devEmum;
-	ComPtr<IMMDevice>             device;
+	ComPtr<IMMDeviceEnumerator> devEmum;
+	ComPtr<IMMDevice> device;
 	ComPtr<IAudioSessionManager2> sessionManager2;
-	ComPtr<IAudioSessionControl>  sessionControl;
+	ComPtr<IAudioSessionControl> sessionControl;
 	ComPtr<IAudioSessionControl2> sessionControl2;
 
-	HRESULT result = CoCreateInstance(__uuidof(MMDeviceEnumerator),
-			nullptr, CLSCTX_INPROC_SERVER,
-			__uuidof(IMMDeviceEnumerator),
-			(void **)&devEmum);
+	HRESULT result = CoCreateInstance(__uuidof(MMDeviceEnumerator), nullptr,
+					  CLSCTX_INPROC_SERVER,
+					  __uuidof(IMMDeviceEnumerator),
+					  (void **)&devEmum);
 	if (FAILED(result))
 		return false;
 
@@ -244,13 +252,13 @@ bool DisableAudioDucking(bool disable)
 		return false;
 
 	result = device->Activate(__uuidof(IAudioSessionManager2),
-			CLSCTX_INPROC_SERVER, nullptr,
-			(void **)&sessionManager2);
+				  CLSCTX_INPROC_SERVER, nullptr,
+				  (void **)&sessionManager2);
 	if (FAILED(result))
 		return false;
 
 	result = sessionManager2->GetAudioSessionControl(nullptr, 0,
-			&sessionControl);
+							 &sessionControl);
 	if (FAILED(result))
 		return false;
 
@@ -262,17 +270,6 @@ bool DisableAudioDucking(bool disable)
 	return SUCCEEDED(result);
 }
 
-uint64_t CurrentMemoryUsage()
-{
-	PROCESS_MEMORY_COUNTERS pmc = {};
-	pmc.cb = sizeof(pmc);
-
-	if (!GetProcessMemoryInfo(GetCurrentProcess(), &pmc, sizeof(pmc)))
-		return 0;
-
-	return (uint64_t)pmc.WorkingSetSize;
-}
-
 struct RunOnceMutexData {
 	WinHandle handle;
 
@@ -299,7 +296,7 @@ RunOnceMutex &RunOnceMutex::operator=(RunOnceMutex &&rom)
 	return *this;
 }
 
-RunOnceMutex GetRunOnceMutex(bool &already_running)
+RunOnceMutex CheckIfAlreadyRunning(bool &already_running)
 {
 	string name;
 
@@ -307,10 +304,13 @@ RunOnceMutex GetRunOnceMutex(bool &already_running)
 		name = "OBSStudioCore";
 	} else {
 		char path[500];
+		char absPath[512];
 		*path = 0;
+		*absPath = 0;
 		GetConfigPath(path, sizeof(path), "");
+		os_get_abs_path(path, absPath, sizeof(absPath));
 		name = "OBSStudioPortable";
-		name += path;
+		name += absPath;
 	}
 
 	BPtr<wchar_t> wname;
@@ -334,3 +334,163 @@ RunOnceMutex GetRunOnceMutex(bool &already_running)
 	RunOnceMutex rom(h ? new RunOnceMutexData(h) : nullptr);
 	return rom;
 }
+
+struct MonitorData {
+	const wchar_t *id;
+	MONITORINFOEX info;
+	bool found;
+};
+
+static BOOL CALLBACK GetMonitorCallback(HMONITOR monitor, HDC, LPRECT,
+					LPARAM param)
+{
+	MonitorData *data = (MonitorData *)param;
+
+	if (GetMonitorInfoW(monitor, &data->info)) {
+		if (wcscmp(data->info.szDevice, data->id) == 0) {
+			data->found = true;
+			return false;
+		}
+	}
+
+	return true;
+}
+
+#if QT_VERSION < QT_VERSION_CHECK(6, 4, 0)
+#define GENERIC_MONITOR_NAME QStringLiteral("Generic PnP Monitor")
+
+QString GetMonitorName(const QString &id)
+{
+	MonitorData data = {};
+	data.id = (const wchar_t *)id.utf16();
+	data.info.cbSize = sizeof(data.info);
+
+	EnumDisplayMonitors(nullptr, nullptr, GetMonitorCallback,
+			    (LPARAM)&data);
+	if (!data.found) {
+		return GENERIC_MONITOR_NAME;
+	}
+
+	UINT32 numPath, numMode;
+	if (GetDisplayConfigBufferSizes(QDC_ONLY_ACTIVE_PATHS, &numPath,
+					&numMode) != ERROR_SUCCESS) {
+		return GENERIC_MONITOR_NAME;
+	}
+
+	std::vector<DISPLAYCONFIG_PATH_INFO> paths(numPath);
+	std::vector<DISPLAYCONFIG_MODE_INFO> modes(numMode);
+
+	if (QueryDisplayConfig(QDC_ONLY_ACTIVE_PATHS, &numPath, paths.data(),
+			       &numMode, modes.data(),
+			       nullptr) != ERROR_SUCCESS) {
+		return GENERIC_MONITOR_NAME;
+	}
+
+	DISPLAYCONFIG_TARGET_DEVICE_NAME target;
+	bool found = false;
+
+	paths.resize(numPath);
+	for (size_t i = 0; i < numPath; ++i) {
+		const DISPLAYCONFIG_PATH_INFO &path = paths[i];
+
+		DISPLAYCONFIG_SOURCE_DEVICE_NAME s;
+		s.header.type = DISPLAYCONFIG_DEVICE_INFO_GET_SOURCE_NAME;
+		s.header.size = sizeof(s);
+		s.header.adapterId = path.sourceInfo.adapterId;
+		s.header.id = path.sourceInfo.id;
+
+		if (DisplayConfigGetDeviceInfo(&s.header) == ERROR_SUCCESS &&
+		    wcscmp(data.info.szDevice, s.viewGdiDeviceName) == 0) {
+			target.header.type =
+				DISPLAYCONFIG_DEVICE_INFO_GET_TARGET_NAME;
+			target.header.size = sizeof(target);
+			target.header.adapterId = path.sourceInfo.adapterId;
+			target.header.id = path.targetInfo.id;
+			found = DisplayConfigGetDeviceInfo(&target.header) ==
+				ERROR_SUCCESS;
+			break;
+		}
+	}
+
+	if (!found) {
+		return GENERIC_MONITOR_NAME;
+	}
+
+	return QString::fromWCharArray(target.monitorFriendlyDeviceName);
+}
+#endif
+
+/* Based on https://www.winehq.org/pipermail/wine-devel/2008-September/069387.html */
+typedef const char *(CDECL *WINEGETVERSION)(void);
+bool IsRunningOnWine()
+{
+	WINEGETVERSION func;
+	HMODULE nt;
+
+	nt = GetModuleHandleW(L"ntdll");
+	if (!nt)
+		return false;
+
+	func = (WINEGETVERSION)GetProcAddress(nt, "wine_get_version");
+	if (func) {
+		blog(LOG_WARNING, "Running on Wine version \"%s\"", func());
+		return true;
+	}
+
+	return false;
+}
+
+HWND hwnd;
+void TaskbarOverlayInit()
+{
+	hwnd = (HWND)App()->GetMainWindow()->winId();
+}
+
+void TaskbarOverlaySetStatus(TaskbarOverlayStatus status)
+{
+	ITaskbarList4 *taskbarIcon;
+	auto hr = CoCreateInstance(CLSID_TaskbarList, NULL,
+				   CLSCTX_INPROC_SERVER,
+				   IID_PPV_ARGS(&taskbarIcon));
+
+	if (FAILED(hr)) {
+		taskbarIcon->Release();
+		return;
+	}
+
+	hr = taskbarIcon->HrInit();
+
+	if (FAILED(hr)) {
+		taskbarIcon->Release();
+		return;
+	}
+
+	QIcon qicon;
+	switch (status) {
+	case TaskbarOverlayStatusActive:
+		qicon = QIcon::fromTheme("obs-active",
+					 QIcon(":/res/images/active.png"));
+		break;
+	case TaskbarOverlayStatusPaused:
+		qicon = QIcon::fromTheme("obs-paused",
+					 QIcon(":/res/images/paused.png"));
+		break;
+	case TaskbarOverlayStatusInactive:
+		taskbarIcon->SetOverlayIcon(hwnd, nullptr, nullptr);
+		taskbarIcon->Release();
+		return;
+	}
+
+	HICON hicon = nullptr;
+	if (!qicon.isNull()) {
+		Q_GUI_EXPORT HICON qt_pixmapToWinHICON(const QPixmap &p);
+		hicon = qt_pixmapToWinHICON(
+			qicon.pixmap(GetSystemMetrics(SM_CXSMICON)));
+		if (!hicon)
+			return;
+	}
+
+	taskbarIcon->SetOverlayIcon(hwnd, hicon, nullptr);
+	DestroyIcon(hicon);
+	taskbarIcon->Release();
+}
